@@ -166,6 +166,8 @@ gnome_bg_slide_show_finalize (GObject *object)
 
         g_list_foreach (self->priv->stack->head, (GFunc) g_free, NULL);
         g_queue_free (self->priv->stack);
+
+        g_free (self->priv->filename);
 }
 
 static void
@@ -563,6 +565,7 @@ gnome_bg_slide_show_get_current_slide (GnomeBGSlideShow  *self,
  * @frame_number: frame number
  * @width: monitor width
  * @height: monitor height
+ * @progress: (out) (allow-none): slide progress
  * @duration: (out) (allow-none): slide duration
  * @is_fixed: (out) (allow-none): if slide is fixed
  * @file1: (out) (allow-none) (transfer none): first file in slide
@@ -682,6 +685,10 @@ parse_file_contents (GnomeBGSlideShow  *self,
 
                 /* no slides, that's not a slideshow */
                 if (len == 0) {
+                        g_set_error_literal (error,
+                                             G_MARKUP_ERROR,
+                                             G_MARKUP_ERROR_INVALID_CONTENT,
+                                             "file is not a slide show since it has no slides");
                         failed = TRUE;
                 /* one slide, there's no transition */
                 } else if (len == 1) {
