@@ -20,18 +20,37 @@
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 
-#include <gdk/gdkx.h>
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libgnome-desktop/gnome-rr.h>
 
+static const char *
+dpms_mode_to_str (GnomeRRDpmsMode mode)
+{
+	switch (mode) {
+	case GNOME_RR_DPMS_ON:
+		return "on";
+	case GNOME_RR_DPMS_STANDBY:
+		return "standby";
+	case GNOME_RR_DPMS_SUSPEND:
+		return "suspend";
+	case GNOME_RR_DPMS_OFF:
+		return "off";
+	case GNOME_RR_DPMS_UNKNOWN:
+		return "unknown";
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 static void
 print_output (GnomeRROutput *output, const char *message)
 {
 	gsize len = 0;
 	const guint8 *result = NULL;
 	int width_mm, height_mm;
-
+	GnomeRRMode **modes;
+	int i;
 	g_print ("[%s]", gnome_rr_output_get_name (output));
 	if (message)
 		g_print (" (%s)", message);
@@ -41,15 +60,36 @@ print_output (GnomeRROutput *output, const char *message)
 	g_print ("\tprimary: %i\n", gnome_rr_output_get_is_primary (output));
 	g_print ("\tid: %i\n", gnome_rr_output_get_id (output));
 	gnome_rr_output_get_physical_size (output, &width_mm, &height_mm);
-	g_print ("\tdimensions: %ix%i", width_mm, height_mm);
+	g_print ("\tdimensions: %ix%i\n", width_mm, height_mm);
+	g_print ("\tbacklight: %i (min step: %i)\n",
+		 gnome_rr_output_get_backlight (output),
+		 gnome_rr_output_get_min_backlight_step (output));
+
 
 	/* get EDID */
         result = gnome_rr_output_get_edid_data (output, &len);
 	if (result != NULL) {
+		char *vendor, *product, *serial;
+
 		g_print ("\tedid: %" G_GSIZE_FORMAT " bytes [%i:%i:%i:%i]\n",
 			 len, result[0], result[1],
 			 result[2], result[3]);
+		gnome_rr_output_get_ids_from_edid (output, &vendor, &product, &serial);
+		g_print ("\tvendor: '%s' product: '%s' serial: '%s'\n",
+			 vendor, product, serial);
+		g_free (vendor);
+		g_free (product);
+		g_free (serial);
 	}
+
+	modes = gnome_rr_output_list_modes (output);
+	for (i = 0; modes[i] != NULL; ++i) {
+		g_print ("\t mode: %dx%d%s\n",
+			 gnome_rr_mode_get_width (modes[i]),
+			 gnome_rr_mode_get_height (modes[i]),
+			 gnome_rr_mode_get_is_tiled (modes[i]) ? " (tiled)" : "");
+	}
+	g_print ("\n");
 }
 
 static void
@@ -70,6 +110,16 @@ output_connected (GnomeRRScreen *screen, GnomeRROutput *output, gpointer user_da
 	print_output (output, "connected");
 }
 
+static void
+dpms_mode_changed (GnomeRRScreen *screen, GParamSpec *pspec, gpointer user_data)
+{
+	GnomeRRDpmsMode mode;
+
+	gnome_rr_screen_get_dpms_mode (screen, &mode, NULL);
+	g_print ("DPMS mode changed to: %s\n", dpms_mode_to_str (mode));
+	g_print ("\n");
+}
+
 /**
  * main:
  **/
@@ -95,6 +145,7 @@ main (int argc, char *argv[])
 	g_signal_connect (screen, "changed", G_CALLBACK (screen_changed), NULL);
 	g_signal_connect (screen, "output-disconnected", G_CALLBACK (output_disconnected), NULL);
 	g_signal_connect (screen, "output-connected", G_CALLBACK (output_connected), NULL);
+	g_signal_connect (screen, "notify::dpms-mode", G_CALLBACK (dpms_mode_changed), NULL);
 
 	gtk_main ();
 
