@@ -229,12 +229,112 @@ gnome_wall_clock_class_init (GnomeWallClockClass *klass)
 	g_type_class_add_private (gobject_class, sizeof (GnomeWallClockPrivate));
 }
 
+/* Replace 'target' with 'replacement' in the input string. */
+static char *
+string_replace (const char *input,
+                const char *target,
+                const char *replacement)
+{
+	char **pieces = NULL;
+	char *output = NULL;
+
+	pieces = g_strsplit (input, target, -1);
+	output = g_strjoinv (replacement, pieces);
+	g_strfreev (pieces);
+	return output;
+}
+
+/* This function wraps g_date_time_format, replacing colon with the ratio
+ * character as it looks visually better in time strings.
+ */
+static char *
+date_time_format (GDateTime *datetime,
+                  const char *format)
+{
+	char *format_with_colon;
+	char *ret;
+	char *tmp;
+	gboolean is_utf8;
+
+	is_utf8 = g_get_charset (NULL);
+
+	/* First, replace ratio with plain colon before passing it to
+	 * g_date_time_format.  */
+	tmp = string_replace (format, "∶", ":");
+	format_with_colon = g_date_time_format (datetime, tmp);
+	g_free (tmp);
+
+	/* Then, after formatting, replace the plain colon with ratio, and
+	 * prepend it with an LTR marker to force direction. */
+	if (is_utf8)
+		ret = string_replace (format_with_colon, ":", "\xE2\x80\x8E∶");
+	else
+		ret = g_strdup (format_with_colon);
+
+	g_free (format_with_colon);
+	return ret;
+}
+
+/**
+ * gnome_wall_clock_string_for_datetime:
+ *
+ * Returns: (skip): a newly allocated string representing the date & time
+ * passed, with the options applied.
+ */
+char *
+gnome_wall_clock_string_for_datetime (GnomeWallClock      *self,
+				      GDateTime           *now,
+				      GDesktopClockFormat  clock_format,
+				      gboolean             show_weekday,
+				      gboolean             show_full_date,
+				      gboolean             show_seconds)
+{
+	const char *format_string;
+
+	if (clock_format == G_DESKTOP_CLOCK_FORMAT_24H ||
+	    self->priv->ampm_available == FALSE) {
+		if (show_full_date) {
+			/* Translators: This is the time format with full date used
+			   in 24-hour mode. */
+			format_string = show_seconds ? _("%a %b %e, %R:%S")
+				: _("%a %b %e, %R");
+		} else if (show_weekday) {
+			/* Translators: This is the time format with day used
+			   in 24-hour mode. */
+			format_string = show_seconds ? _("%a %R:%S")
+				: _("%a %R");
+		} else {
+			/* Translators: This is the time format without date used
+			   in 24-hour mode. */
+			format_string = show_seconds ? _("%R:%S") : _("%R");
+		}
+	} else {
+		if (show_full_date) {
+			/* Translators: This is a time format with full date used
+			   for AM/PM. */
+			format_string = show_seconds ? _("%a %b %e, %l:%M:%S %p")
+				: _("%a %b %e, %l:%M %p");
+		} else if (show_weekday) {
+			/* Translators: This is a time format with day used
+			   for AM/PM. */
+			format_string = show_seconds ? _("%a %l:%M:%S %p")
+				: _("%a %l:%M %p");
+		} else {
+			/* Translators: This is a time format without date used
+			   for AM/PM. */
+			format_string = show_seconds ? _("%l:%M:%S %p")
+				: _("%l:%M %p");
+		}
+	}
+
+	return date_time_format (now, format_string);
+}
+
 static gboolean
 update_clock (gpointer data)
 {
 	GnomeWallClock   *self = data;
 	GDesktopClockFormat clock_format;
-	const char *format_string;
 	gboolean show_full_date;
 	gboolean show_weekday;
 	gboolean show_seconds;
@@ -252,7 +352,7 @@ update_clock (gpointer data)
 		expiry = g_date_time_add_seconds (now, 1);
 	else
 		expiry = g_date_time_add_seconds (now, 60 - g_date_time_get_second (now));
-  
+
 	if (self->priv->clock_update_id)
 		g_source_remove (self->priv->clock_update_id);
 
@@ -262,57 +362,17 @@ update_clock (gpointer data)
 	self->priv->clock_update_id = g_source_attach (source, NULL);
 	g_source_unref (source);
 
-	if (clock_format == G_DESKTOP_CLOCK_FORMAT_24H ||
-	    self->priv->ampm_available == FALSE) {
-		if (show_full_date) {
-			/* Translators: This is the time format with full date used
-			   in 24-hour mode.
-                           The character between %R and %S is U+2236 RATIO */
-			format_string = show_seconds ? _("%a %b %e, %R∶%S")
-				: _("%a %b %e, %R");
-		} else if (show_weekday) {
-			/* Translators: This is the time format with day used
-			   in 24-hour mode.
-                           The character between %R and %S is U+2236 RATIO */
-			format_string = show_seconds ? _("%a %R∶%S")
-				: _("%a %R");
-		} else {
-			/* Translators: This is the time format without date used
-			   in 24-hour mode.
-                           The character between %R and %S is U+2236 RATIO */
-			format_string = show_seconds ? _("%R∶%S") : _("%R");
-		}
-	} else {
-		if (show_full_date) {
-			/* Translators: This is a time format with full date used
-			   for AM/PM.
-                           The Unicode characters are U+2236 RATIO and
-                           U+2009 THIN SPACE */
-			format_string = show_seconds ? _("%a %b %e, %l∶%M∶%S %p")
-				: _("%a %b %e, %l∶%M %p");
-		} else if (show_weekday) {
-			/* Translators: This is a time format with day used
-			   for AM/PM.
-                           The Unicode characters are U+2236 RATIO and
-                           U+2009 THIN SPACE */
-			format_string = show_seconds ? _("%a %l∶%M∶%S %p")
-				: _("%a %l∶%M %p");
-		} else {
-			/* Translators: This is a time format without date used
-			   for AM/PM.
-                           The Unicode characters are U+2236 RATIO and
-                           U+2009 THIN SPACE */
-			format_string = show_seconds ? _("%l∶%M∶%S %p")
-				: _("%l∶%M %p");
-		}
-	}
-
 	g_free (self->priv->clock_string);
-	self->priv->clock_string = g_date_time_format (now, format_string);
+	self->priv->clock_string = gnome_wall_clock_string_for_datetime (self,
+									 now,
+									 clock_format,
+									 show_weekday,
+									 show_full_date,
+									 show_seconds);
 
 	g_date_time_unref (now);
 	g_date_time_unref (expiry);
-      
+
 	g_object_notify ((GObject*)self, "clock");
 
 	return FALSE;
